//
//  AudioManager.h
//  iDIA
//
//  Created by Stanford Harmonics on 2/13/10.
//  Copyright 2010 Stanford University. All rights reserved.
//

#import <Foundation/Foundation.h>
#import "PitchTracker.h"
#import "RecordingManager.h"
#import "VisualObject.h"
#import "Stk.h"
#import "OnePole.h"
#import "RtAudio.h"


#define MY_SRATE 44100
#define FRAMESIZE BUFFER_SIZE
#define NUMCHANNELS 2
#define NUM_SECONDS 3
#define SCORE_MSG @"scoreChanged"


using namespace stk;

@class RecordingManager, Click;

@interface AudioManager : NSObject {
	
	BOOL started;
	BOOL isMuted, isTracking, shouldStopSinging, playClick;
	long double t;
	float gain;
	int tempo;
	int beatTick;
	float muteAtten;
	PitchTracker * pitchTracker;
	RecordingManager * recordingManager;
	int justTicked;
	RtAudio * dac;
	OnePole * lpf;
	VisualObject * visObj;
}

@property(nonatomic, readonly) BOOL isMuted, isTracking, shouldStopSinging, started;
@property(nonatomic) int justTicked;
@property(nonatomic, readonly) long double t; 
@property(nonatomic, readonly) int tempo, beatTick;
@property(nonatomic, readonly) float muteAtten;
@property(nonatomic, readonly) PitchTracker * pitchTracker;
@property(nonatomic, readonly) RecordingManager * recordingManager;
@property(nonatomic, readonly) VisualObject * visObj;
@property(nonatomic, readonly) OnePole * lpf;
@property(nonatomic, readwrite) BOOL playClick;
@property float gain;

- (void) test;

- (void) startCallback;
- (void) startTracking;
- (void) stopSinging;

//- (void) initRecorder;
- (void) muteOn;
- (void) muteOff;

- (void) incBeatTick;
- (void) applyMuteRampOn:(Float32*)buffer of:(int)numFrames;

+ (AudioManager*) sharedManager;
+ (void) releaseShared;

- (void) notifyChanged;

- (int) getSamplesPerBeat;

- (void) startAt:(long double)time;
- (void) stop;
- (void) pause;
- (void) unpause;
- (BOOL) isStarted;
- (void) incTimeBy:(double)s;

@end
