//
//  AudioManager.mm
//  iDIA
//
//  Created by Stanford Harmonics on 2/13/10.
//  Copyright 2010 Stanford University. All rights reserved.
//

#import "AudioManager.h"
#import "AudioUtility.h"
#import "libxtract.h"
#import <iostream>
#import <set>
#import <utility>
#import <sys/time.h>
#import "Stk.h"

#define NUM_CHANNELS 1
#define MUTE_RATE .02
#define ATTACK_THRESH .004
#define GAIN_THRESH .02


static AudioManager * shared;
//timeval startTime;
double lastEvent[11];
set<int> curOn[11];
bool first = true;

int g_t = 0;
int bangTimer = 0;


int audioCallback( void * buffer, void *inputBuffer, unsigned int numFrames, double streamTime, RtAudioStreamStatus status, void * userData )
{	
	NSAutoreleasePool * pool = [[NSAutoreleasePool alloc] init];
	
	AudioManager * theManager = [AudioManager sharedManager];

	if(theManager.isMuted && theManager.muteAtten <= 0) return 0;
	
	
    SAMPLE * buffi = (SAMPLE *)inputBuffer;
    //SAMPLE * buffo = (SAMPLE *)buffer;
	
	SAMPLE sampFloat = MY_SRATE;
	SAMPLE * sampPtr = &sampFloat;
	SAMPLE result;
	xtract[XTRACT_F0](buffi, numFrames, &sampFloat, &result);
	NSLog(@"%f", result);
	
	
	//long double t = theManager.t;
 	// track pitch
	//[theManager.pitchTracker readBuffer:buffi atTime: t];
	theManager.gain = [AudioUtility averageOfBuffer:buffi of:numFrames];
	theManager.visObj.gain = theManager.gain;
	//NSLog(@"gain: %f, pitch: %f", gain, [[AudioManager sharedManager].pitchTracker curFreq]);
	// NSLog(@"curFreq: %f", [[AudioManager sharedManager].pitchTracker curFreq]);
	// zero out output buffer
	//memset(buffer, 0, 2*numFrames*sizeof(SAMPLE));
	
	
	
	if(!bangTimer && theManager.gain > GAIN_THRESH)
	{
		for(int i = 0; i < numFrames; i++)
		{
			for(int j = 0; j < NUM_CHANNELS; j++)
			{
				float leaky = theManager.lpf->tick( buffi[NUM_CHANNELS*i+j] * buffi[NUM_CHANNELS*i+j] );
				if( leaky > ATTACK_THRESH) 
				{
					//NSLog(@"bang! %f gain: %f", leaky, theManager.gain);
					[theManager.visObj addVelocity:CGPointMake([AudioUtility randfBetween:-.3 andUpper:.3], [AudioUtility randfBetween:-.3 andUpper:.3])];
					bangTimer = 1;
					break;
				}
			}
			if(bangTimer) break;
		}
	}
	else bangTimer++;
	if(bangTimer > 5) bangTimer = 0;
	
	[pool release];
	return 0;
}


void testingFunc()
{
	float mean = 0, vector[] = {.1, .2, .3, .4, -.5, -.4, -.3, -.2, -.1},
	spectrum[10];
    int n, N = 9;
    float argf[4];
	
    argf[0] = 8000.f;
    argf[1] = XTRACT_MAGNITUDE_SPECTRUM;
    argf[2] = 0.f;
    argf[3] = 0.f;
    
    xtract[XTRACT_MEAN]((const float *)&vector, N, NULL, (float *)&mean);
    xtract_init_fft(N, XTRACT_SPECTRUM);
    xtract[XTRACT_SPECTRUM]((const float *)&vector, N, &argf[0], (float *)&spectrum[0]);
	
    printf("\nThe mean of [1, 2, 3, 4, 5, 6, 7, 8, 9, 10] = %.1f\n\n", mean);
    printf("\nResults of xtract_spectrum():\n");
	
    for(n = 0; n < N; n++){
        printf("%.3f\t", spectrum[n]);
    }
    printf("\n");	
}


@implementation AudioManager
		   

@synthesize isMuted, isTracking, shouldStopSinging, started, justTicked, t, tempo, beatTick, muteAtten, pitchTracker, 
	recordingManager, visObj, playClick, gain, lpf;

- (void) test
{
	NSLog(@"test");
}

+ (AudioManager*) sharedManager
{
	if(shared == nil)
	{
		NSLog(@"init shared audio manager");
		shared = [[AudioManager alloc] init];
	}
	
	return shared;
}

+ (void) releaseShared
{
	[shared release];
}

- (BOOL) isStarted
{
	return started;
}

- (void) startAt:(long double)time
{
	started = YES;
	//gettimeofday(&startTime, NULL);
	t = time;
}

- (void) unpause
{
	started = YES;
}

- (void) stop
{
	started = NO;
}

- (void) pause
{
	started = NO;
}

- (id) init
{
	if(self = [super init])
	{
		
		testingFunc();
		dac = new RtAudio();
		if ( dac->getDeviceCount() < 1 ) {
			std::cerr << "\nNo audio devices found!\n";
			exit( 0 );
		}
		
		int inDevice = dac->getDefaultInputDevice();
		int outDevice = dac->getDefaultOutputDevice();
		
		// Scan through devices for various capabilities
		RtAudio::DeviceInfo info = dac->getDeviceInfo( inDevice );
		int maxInputs = 0;
		if ( info.probed == true ) maxInputs = info.inputChannels;
		else {
			cerr << "cannot retrieve audio device information" << endl;
			exit(0);
		}
		
		RtAudio::StreamParameters inParameters;
		inParameters.deviceId = inDevice;
		inParameters.nChannels = NUM_CHANNELS;
		inParameters.firstChannel = 0;
		
		RtAudio::StreamParameters outParameters;
		outParameters.deviceId = outDevice;
		outParameters.nChannels = NUM_CHANNELS;
		outParameters.firstChannel = 0;
		
		unsigned int bufferFrames = FRAMESIZE;
		unsigned int sampleRate = MY_SRATE;
		
		try {
			dac->openStream( &outParameters, &inParameters, RTAUDIO_FLOAT32,
						   sampleRate, &bufferFrames, &audioCallback, NULL );
			dac->startStream();
			cerr << "starting realtime audio..." << endl;
		} 
		catch ( RtError& e ) {
			e.printMessage();
			exit( 0 );
		}
		
		// init vars
		isMuted = NO;
		isTracking = NO;
		shouldStopSinging = NO;
		started = NO;
		playClick = YES;
		justTicked = 0;
		beatTick = 0;
		muteAtten = 1.0;
		pitchTracker = [[PitchTracker alloc] init];
		recordingManager = [[RecordingManager alloc] init];
		lpf = new OnePole();
		lpf->setPole(.99);
		visObj = [[VisualObject alloc] init];
		
	}
	
	return self;
}

- (void) startCallback
{
	/*
	// start the audio layer, registering a callback method 
	bool result = MoAudio::start(audioCallback, NULL); 
	if( !result ) 
	{ 
		// something went wrong 
		NSLog( @"cannot start real-time audio!" ); 
		// bail out 
		return; 
	} 
	 */
}

- (void) setGain:(float)aGain
{
	gain = aGain;
}

- (void) stopSinging
{
	NSLog(@"stop singing!");
	shouldStopSinging = YES;
	isTracking = NO;
	[recordingManager stopPlayback];
	[recordingManager stopRecording];
}

- (void) startTracking
{
	isTracking = YES;
}

- (void) muteOn
{
	isMuted = YES;
	started = NO;
}

- (void) muteOff
{
	isMuted = NO;
	started = YES;
}

- (void) incBeatTick
{
	beatTick++;
}

- (void) applyMuteRampOn:(Float32*)buffer of:(int)numFrames
{
	//NSLog(@"muteAtten: %f", muteAtten);
	for(int i = 0; i < numFrames; i++)
	{
		buffer[2*i] *= muteAtten;
		buffer[2*i+1] *= muteAtten;
	}
	if(isMuted) muteAtten -= MUTE_RATE;
	else muteAtten += MUTE_RATE;
}			

- (int) getSamplesPerBeat
{
	float bps = tempo / 60.0;
	float bps16th = bps * 4;
	return floor( MY_SRATE / bps16th );
}

- (void) notifyChanged
{
	[[NSNotificationCenter defaultCenter] postNotificationName:SCORE_MSG object:self];
}

/*
- (long double) timePassed{
	if(!started) return -1;
	timeval result, now;
	gettimeofday(&now, NULL);
	timeval_subtract(&result, &now, &startTime);
	long double dt = result.tv_sec + result.tv_usec / pow(10, 6);
	return dt;
}
 */

- (void) incTimeBy:(double)s
{
	//NSLog(@"Incrementing %f by %f", t, s);
	t += s;
}
@end
