//
//  AudioUtility.mm
//  iDIA
//
//  Created by Stanford Harmonics on 3/1/10.
//  Copyright 2010 Stanford University. All rights reserved.
//

#import "AudioUtility.h"
#import "PitchTracker.h"

@implementation AudioUtility


//-----------------------------------------------------------------------------
// Name: averageBuffer
// Desc: finds the average value of an array of floats
//-----------------------------------------------------------------------------
+ (float) averageOfBuffer:(SAMPLE*)buffer of:(int)buffer_size
{
	float sum = 0;
	for(int i = 0; i < buffer_size; i++) sum += fabs(buffer[i]);
	return sum/buffer_size;
}

+ (float) PI
{
	return 3.141592653589793;
}

+ (float) mtof:(int)mid
{
	return 440 * pow(2, ((float)(mid-69)/12));
}

+ (float) ftom:(float)freq
{
	return 69 + 12*log(freq/440)/log(2);
}

+ (float) ftoy:(float)freq
{
	float midi = [self ftom:freq];
	int index = (int)(floor(midi - MIN_NOTE));
	float remainder = fmod(midi, floor(midi));
	int octave = index / 12;
	int offset;
	
	switch(index % 12) {
		case 0:
			offset = 0;
			remainder *= .5;
			break;
		case 1:
			offset = 0;
			remainder = remainder * .5 + .5;
			break;
		case 2:
			offset = 1;
			remainder *= .5;
			break;
		case 3:
			offset = 1;
			remainder = remainder * .5 + .5;
			break;
		case 4:
			offset = 2;
			break;
		case 5:
			offset = 3;
			remainder *= .5;
			break;
		case 6:
			offset = 3;
			remainder = remainder * .5 + .5;
			break;
		case 7:
			offset = 4;
			remainder *= .5;
			break;
		case 8:
			offset = 4;
			remainder = remainder * .5 + .5;
			break;
		case 9:
			offset = 5;
			remainder *= .5;
			break;
		case 10:
			offset = 5;
			remainder = remainder * .5 + .5;
			break;
		case 11:
			offset = 6;
			break;
	}
	
	//cout << "midi: " << midi << " octave: " << octave << " offset: " << offset << " remainder: " << remainder << endl;
	
	return (((float)octave * 7 + (float)offset + remainder) / 28) * HEIGHT - HEIGHT / 2;
}


+ (float) ftoy:(float)freq fromMinMIDI:(float)minMIDI toMaxMIDI:(float)maxMIDI
{
	float midiNote = [self ftom:freq];
	return (midiNote - minMIDI)/(maxMIDI - minMIDI);
}

+ (float)randfBetween:(float)a andUpper:(float)b
{
    float diff = b - a;
    return a + ((float)rand() / RAND_MAX)*diff;
}

+ (int)randBetween:(int)a andUpper:(int)b
{
    int diff = b - a;
    return a + (rand() % (diff+1));
}



@end
