//
//  PitchTracker.h
//  iDIA
//
//  Created by Charlie Forkish on 3/1/10.
//  Copyright 2010 Stanford University. All rights reserved.
//

#import <Foundation/Foundation.h>

#define SAMPLE float

// pitch tracker defines
#define MAX_FREQ 2000
#define MIN_FREQ 50
#define MAX_NOTE 95
#define MIN_NOTE 15
#define MIN_LAG (MY_SRATE / MAX_FREQ) 
#define MAX_LAG (MY_SRATE / MIN_FREQ)
#define BUFFER_SIZE (2 * MAX_LAG) // large enough buffer to perform ACF
#define LAG_BUF_SIZE MAX_LAG - MIN_LAG
#define WINDOW_SIZE MAX_LAG
#define NUM_PLOT_POINTS (int)((NUM_SECONDS * (double) MY_SRATE / BUFFER_SIZE) * 1.5)
#define PITCH_HIST_SIZE 20
#define WIDTH 3
#define HEIGHT 2

@interface PitchTracker : NSObject {
	
	SAMPLE * buffer;
	float * lagBuffer;
	NSMutableArray * noteHist;
	float * plotVertices;
	float * colorArr;
	int vertIndex;
	float score;
	int curMidi;
}


@property(nonatomic, readonly) float* plotVertices;
@property(nonatomic, readonly) float score;
@property(nonatomic, readonly) int curMidi;

- (float) avgDifThroughIndex:(int)index;
- (SAMPLE) autoCorrWithLag:(int)lag atTime:(int)t;
- (SAMPLE) difFuncWithLag:(int)lag;
- (void) calculateLagsWithYIN;
- (void) readBuffer:(SAMPLE*)newBuffer atTime:(long double)time;
- (void) detectPitchAtTime:(long double)time;
- (void) updatePlotVertices;
- (void) initPlotVertices;
- (float) curFreq;
- (float) curY;
- (void) incrementScoreOfPitch:(float)pitch;
- (void) setCurMidi:(float)midi;


@end
