//
//  PitchTracker.mm
//  iDIA
//
//  Created by Stanford Harmonics on 3/1/10.
//  Copyright 2010 Stanford University. All rights reserved.
//

#import "PitchTracker.h"
#import "AudioManager.h"
#import "AudioUtility.h"
#import "NoteObject.h"



@implementation PitchTracker

@synthesize plotVertices, score, curMidi;


int t = 0;

- (id) init
{
	if(self = [super init])
	{
		buffer = new SAMPLE[BUFFER_SIZE];
		lagBuffer = new float[LAG_BUF_SIZE];
		noteHist = [[NSMutableArray array] retain];
		plotVertices = new float[2*NUM_PLOT_POINTS];
		vertIndex = 0;
		[self initPlotVertices];
		curMidi = 0;
	}
	
	return self;
}

- (void) initPlotVertices
{
	for(int i = 0; i < NUM_PLOT_POINTS; i++)
	{
		plotVertices[2*i+1] = HEIGHT;
		plotVertices[2*i] = 0;//-1 * ((double)WIDTH / 2) + (((double)i / (NUM_PLOT_POINTS - 1)) * ((double)WIDTH / 2)) ;
		//NSLog(@"%f", plotVertices[2*i]);
	}
}


- (float) avgDifThroughIndex:(int)index
{
	float sum = 0;
	for(int i = 0; i < index; i++) {
		sum += lagBuffer[i];
	}
	return sum / float(index + 1);
}

//-----------------------------------------------------------------------------
// autocorrelation function
//-----------------------------------------------------------------------------
- (SAMPLE) autoCorrWithLag:(int)lag atTime:(int)t 
{
	float sum = 0;
	for( int i = t + 1; i < t + WINDOW_SIZE; i++) {
		sum += buffer[i] * buffer[i + lag];
	}
	return sum;
}

- (SAMPLE) difFuncWithLag:(int)lag
{
	return [self autoCorrWithLag:0 atTime:0] + [self autoCorrWithLag:0 atTime:lag] 
		- 2 * [self autoCorrWithLag:lag atTime:0] * (1 - lag / MAX_LAG);
}

- (void) calculateLagsWithYIN
{
	lagBuffer[0] = [self difFuncWithLag:MIN_LAG];
	for(int i = 1; i < LAG_BUF_SIZE; i++) {
		lagBuffer[i] = [self difFuncWithLag: i + MIN_LAG] / [self avgDifThroughIndex: i];
	}
}

- (void) readBuffer:(SAMPLE*)newBuffer atTime:(long double)time
{
	/*
	// collapse two-channel to one-channel
	for(int i = 0; i < BUFFER_SIZE; i++)
	{
		buffer[i] = newBuffer[2*i];
	}
	 */
	
	[self calculateLagsWithYIN];
	[self detectPitchAtTime:time];
	//[self updatePlotVertices];
	
}


- (float) curFreq
{
	if([noteHist count] == 0) return 0;
	NoteObject * curNote = [noteHist objectAtIndex:0];
	if( !(curNote && curNote.freq > 0 && curNote.freq < 20000) )
	{
		NSLog(@"pitchTracker: no current note");
		return 220;
	}
	return curNote.freq;
}

- (float) curY
{
	return [AudioUtility ftoy:[self curFreq] fromMinMIDI:MIN_NOTE toMaxMIDI:MAX_NOTE];
}

- (void) updatePlotVertices
{
	for(int i = 0; i < NUM_PLOT_POINTS - 1; i++)
	{
		plotVertices[2*i + 1] = plotVertices[2*(i+1) + 1];
		plotVertices[2*i] = plotVertices[2*(i+1)];
	}
	NoteObject * curNote = [noteHist objectAtIndex:0];
	plotVertices[2 * NUM_PLOT_POINTS - 1] = [self curY];
	plotVertices[2 * NUM_PLOT_POINTS-2] = curNote.time;
	//NSLog(@"CurTime: %f", curNote.time);
}

- (void) detectPitchAtTime:(long double)time
{	
	
	float min = 1;
	int pitchLag = 0;
	
	int trigger = 0;
	float threshold = .1;
	for(int i = 0; i < LAG_BUF_SIZE; i++) {
		if(trigger && lagBuffer[i] > threshold) break;
		if(lagBuffer[i] < min) {
			min = lagBuffer[i];
			pitchLag = i + MIN_LAG;
			if(!trigger && min < threshold) trigger = 1;
		}
	}
	
	float pitch = MY_SRATE / (float)pitchLag;
	t++;
	
	// store detected pitch
	[noteHist insertObject:[[[NoteObject alloc] initWithFreq:pitch andTime:time] autorelease] atIndex:0];
	if(curMidi) [self incrementScoreOfPitch:pitch];
	
	return;
}

- (void) incrementScoreOfPitch:(float) pitch
{
	float deviation = fabs([AudioUtility ftom:pitch] - curMidi);
	//NSLog(@"deviation: %f, pitch: %f, midi: %f", deviation, [AudioUtility ftom:pitch], curMidi);
	if(deviation < 2.0) score += 100 * (1 - deviation);
}


- (void) setCurMidi:(float)midi
{
	curMidi = midi;
}




@end
