//
//  RecordingManager.mm
//  iDIA
//
//  Created by Charlie Forkish on 3/7/10.
//  Copyright 2010 Stanford University. All rights reserved.
//

#import "RecordingManager.h"


@implementation RecordingManager

@synthesize isRecording, isPlaying, recordingArmed, playbackArmed;

- (id) init
{
	if(self = [super init])
	{
		// init STK
		Stk::setSampleRate(MY_SRATE);
		
		// init file reader/writer
		recorder = new FileWrite();
		reader = new FileRead();
		recordBuffer = new StkFrames(BUFFER_SIZE, 1);
		readBuffer = new StkFrames(BUFFER_SIZE, 1);
		isPlaying = NO;
		isRecording = NO;
		recordingArmed = NO;
		playbackArmed = NO;
	}
	
	return self;
}

- (void) armRecording
{
	recordingArmed = YES;
}

- (void) armPlayback
{
	playbackArmed = YES;
}

- (void) startRecording
{
	if(isRecording) return;
	NSLog(@"start recording");
	isRecording = YES;
	
	// create new file in "Documents" directory of app's sandbox
	NSString *file = @"/Documents/test1.wav";
	NSString *path = [NSHomeDirectory() stringByAppendingString: file];
	NSLog(@"Start Recording To: %@", path);
	const char *cPath = [path UTF8String];
	recorder->open(cPath, 1, FileWrite::FILE_WAV, Stk::STK_SINT16);
}

- (void) stopRecording
{
	if(!isRecording) return;
	NSLog(@"stop recording");
	isRecording = NO;
	recorder->close();
}

- (void) startPlayback
{
	if(isPlaying) return;
	NSLog(@"start playback");
	isPlaying = YES;
	
	// open file created by "startRecording"
	NSString *file = @"/Documents/test1.wav";
	NSString *path = [NSHomeDirectory() stringByAppendingString: file];
	const char *cPath = [path UTF8String];
	reader->open(cPath);
	
	// store length of file
	fileSize = reader->fileSize();
	fileIndex = 0;
}

- (void) stopPlayback
{
	if(!isPlaying) return;
	NSLog(@"stop playback");
	isPlaying = NO;
	reader->close();
}

/* this is the function that writes the buffers to file */
- (void) recordBufferIn:(SAMPLE*)buffer
{
	// copy stereo buffer into mono StkFrames object
	for(int i = 0; i < FRAMESIZE; i++)
		(*recordBuffer)[i] = buffer[2*i];
	
	// write to file
	recorder->write(*recordBuffer);
}

/* this function reads a buffer of samples from the file into "buffer" */
- (void) playBuffer:(SAMPLE*)buffer
{
	// read file into StkFrames object
	reader->read(*readBuffer, fileIndex);
	
	// copy data from frames object into buffer
	for(int i = 0; i < FRAMESIZE; i++)
	{
		buffer[2*i] += (*readBuffer)[i];
		buffer[2*i+1] += (*readBuffer)[i];
	}
	
	// increment index of location in file to read from 
	fileIndex += FRAMESIZE;
	
	// if the file's done, stop playback
	if(fileIndex >= fileSize) 
	{
		[self stopPlayback];
		NSLog(@"eof: %d", fileSize);
	}
}

@end
