/*
 * Original Windows comment:
 * "This code was created by Jeff Molofee 2000
 * A HUGE thanks to Fredric Echols for cleaning up
 * and optimizing the base code, making it more flexible!
 * If you've found this code useful, please let me know.
 * Visit my site at nehe.gamedev.net"
 * 
 * Cocoa port by Bryan Blackburn 2002; www.withay.com
 */

/* OpenGLViewController.m */

#import "OpenGLViewController.h"
#import "AudioManager.h"

@interface OpenGLViewController (InternalMethods)
- (void) setupRenderTimer;
- (void) updateGLView:(NSTimer *)timer;
- (void) createFailed;
@end

@implementation OpenGLViewController

- (void) setup
{
	[AudioManager sharedManager];
	
	NSRect mainDisplayRect, viewRect;
	
	// Create a screen-sized window on the display you want to take over
	// Note, mainDisplayRect has a non-zero origin if the key window is on a secondary display
	mainDisplayRect = [[NSScreen mainScreen] frame];
	fullScreenWindow = [[NSWindow alloc] initWithContentRect:mainDisplayRect styleMask:NSBorderlessWindowMask 
													 backing:NSBackingStoreBuffered defer:YES];
	
	// Set the window level to be above the menu bar
	[fullScreenWindow setLevel:NSMainMenuWindowLevel+1];
	
	// Perform any other window configuration you desire
	[fullScreenWindow setOpaque:YES];
	[fullScreenWindow setHidesOnDeactivate:YES];
	
	// Create a view with a double-buffered OpenGL context and attach it to the window
	// By specifying the non-fullscreen context as the shareContext, we automatically inherit the OpenGL objects (textures, etc) it has defined
	viewRect = NSMakeRect(0.0, 0.0, mainDisplayRect.size.width, mainDisplayRect.size.height);
	fullScreenView = [ [ OpenGLView alloc ] initWithFrame:viewRect colorBits:16 depthBits:16 fullscreen:FALSE ];
	[fullScreenWindow setContentView:fullScreenView];
	
	// Show the window
	[fullScreenWindow makeKeyAndOrderFront:self];
	
	// Set the scene with the full-screen viewport and viewing transformation
	//[scene setViewportRect:viewRect];
	
	// Assign the view's MainController to self
	//[fullScreenView setMainController:self];
	

	// Start playing the animation
	//[fullScreenView startAnimation];
	
	
	[ self setupRenderTimer ];
	
	/*
	/////=============
	//[ NSApp setDelegate:self ];   // We want delegate notifications
	renderTimer = nil;
	[ glWindow makeFirstResponder:self ];
	glView = [ [ OpenGLView alloc ] initWithFrame:[glWindow frame] colorBits:16 depthBits:16 fullscreen:FALSE ];
	
	if(glView != nil )
	{
		NSLog(@"initialized OpenGLView");
		[ glWindow setContentView:glView ];
		[ glWindow makeKeyAndOrderFront:self ];
		[ self setupRenderTimer ];
	}
	else
		[ self createFailed ];	
	 */
}

- (id) init
{
	if(self = [super init])
	{
		[self setup];
	}
	return self;
}

- (void) awakeFromNib
{  
	[self setup];
}  


/*
 * Setup timer to update the OpenGL view.
 */
- (void) setupRenderTimer
{
   NSTimeInterval timeInterval = 0.005;

   renderTimer = [ [ NSTimer scheduledTimerWithTimeInterval:timeInterval
                             target:self
                             selector:@selector( updateGLView: )
                             userInfo:nil repeats:YES ] retain ];
   [ [ NSRunLoop currentRunLoop ] addTimer:renderTimer
                                  forMode:NSEventTrackingRunLoopMode ];
   [ [ NSRunLoop currentRunLoop ] addTimer:renderTimer
                                  forMode:NSModalPanelRunLoopMode ];
}


/*
 * Called by the rendering timer.
 */
- (void) updateGLView:(NSTimer *)timer
{
   if( fullScreenView != nil )
      [ fullScreenView drawRect:[ fullScreenView frame ] ];
}  


/*
 * Handle key presses
 */
- (void) keyDown:(NSEvent *)theEvent
{
   unichar unicodeKey;

   unicodeKey = [ [ theEvent characters ] characterAtIndex:0 ];
   switch( unicodeKey )
   {
      // Handle key presses here
   }
}


/*
 * Set full screen.
 */
- (IBAction)setFullScreen:(id)sender
{
	/*
	if([glView isFullScreen])
		[glView	exitFullScreenModeWithOptions:nil];
	else 
		[glView enterFullScreenMode:[NSScreen mainScreen] withOptions:nil];
	 */
	
	/*
	if([glView isFullScreen])
		[glView	clearDrawable];
	else 
		[glView enterFullScreenMode:[NSScreen mainScreen] withOptions:nil];
	 */

	/*
   [ glWindow setContentView:nil ];
   if( [ glView isFullScreen ] )
   {
      if( ![ glView setFullScreen:FALSE inFrame:[ glWindow frame ] ] )
         [ self createFailed ];
      else
         [ glWindow setContentView:glView ];
   }
   else
   {
      if( ![ glView setFullScreen:TRUE
                    inFrame:NSMakeRect( 0, 0, 800, 600 ) ] )
         [ self createFailed ];
   }
	 */
}


/*
 * Called if we fail to create a valid OpenGL view
 */
- (void) createFailed
{
   NSWindow *infoWindow;

   infoWindow = NSGetCriticalAlertPanel( @"Initialization failed",
                                         @"Failed to initialize OpenGL",
                                         @"OK", nil, nil );
   [ NSApp runModalForWindow:infoWindow ];
   [ infoWindow close ];
   [ NSApp terminate:self ];
}


/* 
 * Cleanup
 */
- (void) dealloc
{
   //[ glWindow release ]; 
   [ fullScreenView release ];
   if( renderTimer != nil && [ renderTimer isValid ] )
      [ renderTimer invalidate ];
	[super dealloc];
}

@end
