//
//  VisualObject.m
//  VisualizeMe
//
//  Created by Charlie Forkish on 5/17/10.
//  Copyright 2010 Stanford University. All rights reserved.
//

#import "VisualObject.h"

#define VELOCITY_FACTOR 0.004
#define MAX_VELOCITY 1.0

@implementation VisualObject

@synthesize position, velocity, gain, size;

- (id) init
{
	if(self = [super init])
	{
		position = CGPointZero;
		velocity = CGPointZero;
	}
	
	return self;
}

- (void) setVelocity:(CGPoint)newVelocity
{
	velocity = newVelocity;
}

- (void) addVelocity:(CGPoint)additionalVelocity
{
	velocity.x += additionalVelocity.x;
	if(velocity.x > MAX_VELOCITY)
		velocity.x -= 2*additionalVelocity.x;
	velocity.y += additionalVelocity.y;
	if(velocity.y > MAX_VELOCITY)
		velocity.y -= 2*additionalVelocity.y;
}

- (void) setPosition:(CGPoint)newPosition
{
	position = newPosition;
}

- (void) setGain:(float)newGain
{
	gain = newGain;
}

- (CGPoint) nextPoint
{
	return CGPointMake(position.x + VELOCITY_FACTOR * velocity.x, position.y + VELOCITY_FACTOR * velocity.y);
}

- (void) stepInBounds:(NSRect)bounds
{
	size = gain;
	CGPoint newPoint = [self nextPoint];
	
	/*
	if(newPoint.x < 0 || newPoint.x > bounds.size.width)
	{
		velocity.x = -1.0*velocity.x;
		newPoint = [self nextPoint];
	} else if(newPoint.y < 0 || newPoint.y > bounds.size.height)
	{
		velocity.y = -1.0*velocity.y;
		newPoint = [self nextPoint];
	}
	 */
	 
	if(newPoint.x - size < -1.0 || newPoint.x + size > 1.0)
	{
		velocity.x = -1.0*velocity.x;
		newPoint = [self nextPoint];
	} else if(newPoint.y - size < -1.0 || newPoint.y + size > 1.0)
	{
		velocity.y = -1.0*velocity.y;
		newPoint = [self nextPoint];
	}
	//NSLog(@"%f, %f", velocity.x, velocity.y);
	
	velocity.x *= .995;
	velocity.y *= .995;
	[self setPosition:newPoint];
}


@end
